`timescale 1ns / 1ns

module codec_clocks_generator(CLK, RST,
                              MCLK, BICK, LRCK);

   input CLK;
   input RST;

   wire  CLK;
   wire  RST;

   output MCLK;
   output BICK;
   output LRCK;

   wire   MCLK; 
   wire   BICK; 
   wire   LRCK; 

   reg [0:7] divider;

   reg [0:3] pos_counter, next_pos_counter;
   reg [0:3] neg_counter, next_neg_counter;
   reg 		 pos_strobe, next_pos_strobe;
   reg 		 neg_strobe, next_neg_strobe;

   reg 		 prev_mclk;
   wire 	 rising_mclk;

   assign 	 rising_mclk = !prev_mclk && MCLK;
   
   assign    MCLK = pos_strobe | neg_strobe;
   assign    BICK = divider[6];
   assign    LRCK = divider[0];

   always @(posedge CLK)
	 begin
		if(RST)
		  prev_mclk <= 0;
		else
		  prev_mclk <= MCLK;
	 end



   
   always @(posedge CLK)
	 begin
		if(RST)
		  begin
			 pos_counter <= 0;
			 pos_strobe <= 0;
		  end
		else
		  begin
			 pos_counter <= next_pos_counter;
			 pos_strobe <= next_pos_strobe;
		  end
	 end // always @ (posedge CLK)

   always @(negedge CLK)
	 begin
		if(RST)
		  begin
			 neg_counter <= 4'h4;
			 neg_strobe <= 0;
		  end
		else
		  begin
			 neg_counter <= next_neg_counter;
			 neg_strobe <= next_neg_strobe;
		  end
	 end // always @ (negedge CLK)


   always @(*)
	 begin
		next_pos_counter = pos_counter + 1;
		next_pos_strobe = pos_strobe;
		case(pos_counter)
		  4'b0000: next_pos_strobe = 1;
		  4'b0010: next_pos_strobe = 0;
		  4'b1000: next_pos_counter = 0;
		endcase // case(neg_counter)
	 end

   always @(*)
	 begin
		next_neg_counter = neg_counter + 1;
		next_neg_strobe = neg_strobe;
		case(neg_counter)
		  4'b0000: next_neg_strobe = 1;
		  4'b0010: next_neg_strobe = 0;
		  4'b1000: next_neg_counter = 0;
		endcase // case(neg_counter)
	 end
   

		
   always @(posedge CLK)
     begin
        if(RST)
          divider <= 0;
        else
          divider <= divider + rising_mclk;
     end


endmodule // codec_clocks
