`timescale 1ns / 1ns

module mixer(CLK, RST,
			 LEFT_CHANNEL, RIGHT_CHANNEL, OUTPUT_VALID_STROBE,

			 STRING0, STRING0_VALID_STROBE,
			 STRING1, STRING1_VALID_STROBE,
			 STRING2, STRING2_VALID_STROBE,
			 STRING3, STRING3_VALID_STROBE);

   parameter     STATE_WAITING_FOR_INPUTS = 0;
   parameter 	 STATE_ALL_INPUTS_LATCHED = 1;
   parameter 	 STATE_OUTPUTS_LATCHED    = 2;
   
   input         CLK;
   input 		 RST;

   wire 		 CLK;
   wire 		 RST;
   
   output [0:15] LEFT_CHANNEL;
   output [0:15] RIGHT_CHANNEL;
   output 		 OUTPUT_VALID_STROBE;
   
   reg [0:15] 	 LEFT_CHANNEL, next_LEFT_CHANNEL;
   reg [0:15] 	 RIGHT_CHANNEL, next_RIGHT_CHANNEL;
   wire 		 OUTPUT_VALID_STROBE;
         

   input [0:15]  STRING0;
   input 		 STRING0_VALID_STROBE;
   input [0:15]  STRING1;
   input 		 STRING1_VALID_STROBE;
   input [0:15]  STRING2;
   input 		 STRING2_VALID_STROBE;
   input [0:15]  STRING3;
   input 		 STRING3_VALID_STROBE;
   

   
   wire [0:15] 	 STRING0;
   wire 		 STRING0_VALID_STROBE;
   wire [0:15] 	 STRING1;
   wire 		 STRING1_VALID_STROBE;
   wire [0:15] 	 STRING2;
   wire 		 STRING2_VALID_STROBE;
   wire [0:15] 	 STRING3;
   wire 		 STRING3_VALID_STROBE;
   

   reg [0:1] 	 state, next_state;
   

   //
   //
   // STUDENT CODE STARTS HERE
   //
   //

   reg 			 string0_latched, next_string0_latched;
   reg [0:15] 	 string0_buffer, next_string0_buffer;
   
   reg 			 string1_latched, next_string1_latched;
   reg [0:15] 	 string1_buffer, next_string1_buffer;
   
   reg 			 string2_latched, next_string2_latched;
   reg [0:15] 	 string2_buffer, next_string2_buffer;
   
   reg 			 string3_latched, next_string3_latched;
   reg [0:15] 	 string3_buffer, next_string3_buffer;
   

   assign 		 OUTPUT_VALID_STROBE = (state == STATE_OUTPUTS_LATCHED);
   
   
   always @(posedge CLK)
	 begin
		if(RST)
		  begin
			 LEFT_CHANNEL <= 0;
			 RIGHT_CHANNEL <= 0;
			 
			 string0_latched <= 0;
			 string0_buffer <= 0;
			 string1_latched <= 0;
			 string1_buffer <= 0;
			 string2_latched <= 0;
			 string2_buffer <= 0;
			 string3_latched <= 0;
			 string3_buffer <= 0;

			 state <= STATE_WAITING_FOR_INPUTS;
			 
		  end
		else
		  begin
			 LEFT_CHANNEL <= next_LEFT_CHANNEL;
			 RIGHT_CHANNEL <= next_RIGHT_CHANNEL;
			 
			 string0_latched <= next_string0_latched;
			 string0_buffer <= next_string0_buffer;
			 string1_latched <= next_string1_latched;
			 string1_buffer <= next_string1_buffer;
			 string2_latched <= next_string2_latched;
			 string2_buffer <= next_string2_buffer;
			 string3_latched <= next_string3_latched;
			 string3_buffer <= next_string3_buffer;

			 state <= next_state;

		  end // else: !if(RST)
	 end // always @ (posedge CLK)
   


   // the simple state machine
   always @(*)
	 begin
		next_state = state;
		case(state)
		  STATE_WAITING_FOR_INPUTS:
			   if(string0_latched && string1_latched &&
				  string2_latched && string3_latched) 
				 next_state = STATE_ALL_INPUTS_LATCHED;

		  STATE_ALL_INPUTS_LATCHED:
			next_state = STATE_OUTPUTS_LATCHED;
	
		  STATE_OUTPUTS_LATCHED:
			next_state = STATE_WAITING_FOR_INPUTS;
		endcase // case(state)
	 end // always @ (*)
   
   always @(*)
	 begin
		next_string0_latched = string0_latched;
		next_string0_buffer  = string0_buffer;
		next_string1_latched = string1_latched;
		next_string1_buffer  = string1_buffer;
		next_string2_latched = string2_latched;
		next_string2_buffer  = string2_buffer;
		next_string3_latched = string3_latched;
		next_string3_buffer  = string3_buffer;

		case(state)
		  STATE_WAITING_FOR_INPUTS:
			begin
			   if(STRING0_VALID_STROBE)
				 begin
					next_string0_buffer = STRING0;
					next_string0_latched = 1;
				 end
			   
			   if(STRING1_VALID_STROBE)
				 begin
					next_string1_buffer = STRING1;
					next_string1_latched = 1;
				 end
			   
			   if(STRING2_VALID_STROBE)
				 begin
					next_string2_buffer = STRING2;
					next_string2_latched = 1;
				 end
			   
			   if(STRING3_VALID_STROBE)
				 begin
					next_string3_buffer = STRING3;
					next_string3_latched = 1;
				 end
			end // case: STATE_WAITING_FOR_INPUTS
		  STATE_OUTPUTS_LATCHED:
			begin
			   next_string0_latched = 0;
			   next_string1_latched = 0;
			   next_string2_latched = 0;
			   next_string3_latched = 0;
			end
		endcase // case(state)
	 end // always @ (*)
   


   
   
   wire [0:19] left_channel_sum;
   wire [0:19] right_channel_sum;

   
   assign left_channel_sum =
		  {string0_buffer[0], string0_buffer, 3'b000}     +  // 8 times string 0
		  {{2{string1_buffer[0]}}, string1_buffer, 2'b00} +  // 4 times string 1
		  {{4{string1_buffer[0]}}, string1_buffer}        +  // 1 times string 1
		  {{3{string2_buffer[0]}}, string2_buffer, 1'b0}  +  // 2 times string 2
		  {{4{string2_buffer[0]}}, string2_buffer};          // 1 times string 2
   
   assign right_channel_sum =
		  {string3_buffer[0], string3_buffer, 3'b000}     +  // 8 times string 3
		  {{2{string2_buffer[0]}}, string2_buffer, 2'b00} +  // 4 times string 2
		  {{4{string2_buffer[0]}}, string2_buffer}        +  // 1 times string 2
		  {{3{string1_buffer[0]}}, string1_buffer, 1'b0}  +  // 2 times string 1
		  {{4{string1_buffer[0]}}, string1_buffer};          // 1 times string 1
   

   
   always @(*)
	 begin
		next_LEFT_CHANNEL = LEFT_CHANNEL;
		next_RIGHT_CHANNEL = RIGHT_CHANNEL;
	
		if(state == STATE_ALL_INPUTS_LATCHED)
		  begin
			 next_LEFT_CHANNEL = left_channel_sum[0:15];
			 next_RIGHT_CHANNEL = right_channel_sum[0:15];
		  end
	 end // always @ (*)

   
endmodule // mixer

			 
