`timescale 1ns / 1ns
 
module string_simulation(CLK, 
                         RST,

                         CTRL_EN,
                         CTRL_ADDR,
                         CTRL_DATA_IN,
                         CTRL_DATA_OUT,
                         CTRL_RNW,
                         CTRL_ACK,
                         
                         MCLK, 
                         BICK, 
                         LRCK,
				         SDTI1, 
                         SDTI2, 
                         SDTI3, 
                         SDTI4,
				         SDTO, 
                         PDN);

   
   input          CLK;
   input          RST;
   
   wire           CLK;
   wire           RST;

   
   // control lines
   input          CTRL_EN;
   input [0:7]    CTRL_ADDR;
   input [0:31]   CTRL_DATA_IN;
   output [0:31]  CTRL_DATA_OUT;
   input          CTRL_RNW;
   output         CTRL_ACK;
   
   wire           CTRL_EN;
   wire [0:7]     CTRL_ADDR;
   wire [0:31]    CTRL_DATA_IN;
   wire [0:31]    CTRL_DATA_OUT;
   wire           CTRL_RNW;
   wire           CTRL_ACK;
  

   // codec io lines
   output         MCLK;
   output         BICK;
   output         LRCK;
   output         SDTI1;
   output         SDTI2;
   output         SDTI3;
   output         SDTI4;
   input          SDTO;
   output         PDN;
   
   wire           MCLK;
   wire           BICK;
   wire           LRCK;
   wire           SDTI1;
   wire           SDTI2;
   wire           SDTI3;
   wire           SDTI4;
   wire           SDTO;
   wire           PDN;
   



   
   // codec controller signals
   wire           cc_clk;
   wire           cc_rst;
   wire           cc_master_enable;
   wire           cc_ready_strobes_enable;
   wire [0:31]    cc_received_sample;
   wire [0:127]   cc_send_sample;
   wire [0:7]     cc_send_sample_enables;
   wire           cc_ready_strobe;
   wire           cc_mclk;
   wire           cc_bick;
   wire           cc_lrck;
   wire           cc_sdti1;
   wire           cc_sdti2;
   wire           cc_sdti3;
   wire           cc_sdti4;
   wire           cc_sdto;
   wire           cc_pdn;



   // string controller signals
   wire           sc_clk;
   wire           sc_rst;
   wire           sc_trigger_sample_strobe;
   wire [0:3]     sc_pluck_strobes;
   wire [0:35]    sc_pluck_locations;
   wire [0:63]    sc_pluck_values;
   wire [0:35]    sc_sizes;
   wire [0:31]    sc_pickup_locs;
   wire [0:63]    sc_samples;
   wire [0:3]     sc_sample_valid_strobes;

   // simulation control signals
   wire           ctrl_rst;
   wire           ctrl_clk;
   // don't bother making the wires for the straight
   // passthroughs which are the control signals from outside
   wire           ctrl_master_enable;
   wire [0:35]    ctrl_string_lengths;
   wire [0:31]    ctrl_pickup_locations;
   wire [0:63]    ctrl_pluck_values;
   wire [0:35]    ctrl_pluck_locations;
   wire [0:3]     ctrl_pluck_strobes;
   

   // mixer signals
   wire           mixer_clk;
   wire           mixer_rst;
   wire [0:15]    mixer_left_channel;
   wire [0:15]    mixer_right_channel;
   wire           mixer_output_valid_strobe;
   wire           mixer_string0_enable;
   wire           mixer_string1_enable;
   wire           mixer_string2_enable;
   wire           mixer_string3_enable;
   wire [0:15]    mixer_string0;
   wire [0:15]    mixer_string1;
   wire [0:15]    mixer_string2;
   wire [0:15]    mixer_string3;
   
   
   // assigns for this module's ios
   assign         MCLK = cc_mclk;
   assign         BICK = cc_bick;
   assign         LRCK = cc_lrck;
   assign         SDTI1 = cc_sdti1;
   assign         SDTI2 = cc_sdti2;
   assign         SDTI3 = cc_sdti3;
   assign         SDTI4 = cc_sdti4;
   assign         PDN = cc_pdn;
   
   


   // assigns for the codec
   assign         cc_clk = CLK;
   assign         cc_rst = RST;
   assign         cc_master_enable = ctrl_master_enable;
   assign         cc_ready_strobes_enable = 1;
   assign         cc_send_sample = {
							        mixer_left_channel,
							        mixer_right_channel,
							        cc_received_sample,
							        sc_samples};
   assign         cc_send_sample_enables = {{2{mixer_output_valid_strobe}},
									        {2{cc_ready_strobe}},
									        sc_sample_valid_strobes};
   assign         cc_sdto = SDTO;
   




   
   
   // assigns for the string controller
   assign         sc_clk = CLK;
   assign         sc_rst = RST;
   assign         sc_trigger_sample_strobe = cc_ready_strobe;
   assign         sc_pluck_strobes = ctrl_pluck_strobes;
   assign         sc_pluck_locations = ctrl_pluck_locations;
   assign         sc_pluck_values = ctrl_pluck_values;
   assign         sc_sizes = ctrl_string_lengths;
   assign         sc_pickup_locs = ctrl_pickup_locations;



   assign         ctrl_rst = RST;
   assign         ctrl_clk = CLK;
   

   
   assign         mixer_clk = CLK;
   assign         mixer_rst = RST;
   assign         mixer_string0_enable = sc_sample_valid_strobes[0];
   assign         mixer_string1_enable = sc_sample_valid_strobes[1];
   assign         mixer_string2_enable = sc_sample_valid_strobes[2];
   assign         mixer_string3_enable = sc_sample_valid_strobes[3];
   assign         mixer_string0 = sc_samples[0:15];
   assign         mixer_string1 = sc_samples[16:31];
   assign         mixer_string2 = sc_samples[32:47];
   assign         mixer_string3 = sc_samples[48:63];
   


   
   codec_controller cc
	 (
	  .CLK(cc_clk),
	  .RST(cc_rst),
	  .MASTER_ENABLE(cc_master_enable),
      .READY_STROBES_ENABLE(cc_ready_strobes_enable),
	  .RECEIVED_SAMPLE(cc_received_sample),
	  .SEND_SAMPLE(cc_send_sample),
	  .SEND_SAMPLE_ENABLES(cc_send_sample_enables),
	  .READY_STROBE(cc_ready_strobe),
	  .MCLK(cc_mclk),
	  .BICK(cc_bick),
	  .LRCK(cc_lrck),
	  .SDTI1(cc_sdti1),
	  .SDTI2(cc_sdti2),
	  .SDTI3(cc_sdti3),
	  .SDTI4(cc_sdti4),
	  .SDTO(cc_sdto),
	  .PDN(cc_pdn)
	  );


   string_control sc0
	 (
	  .CLK(sc_clk),
	  .RST(sc_rst),
	  .TRIGGER_SAMPLE_STROBE(sc_trigger_sample_strobe),
	  .PLUCK_STROBE(sc_pluck_strobes[0]),
	  .PLUCK_LOCATION(sc_pluck_locations[0:8]),
	  .PLUCK_VALUE(sc_pluck_values[0:15]),
	  .SIZE(sc_sizes[0:8]),
	  .PICKUP_LOC(sc_pickup_locs[0:7]),
	  .SAMPLE(sc_samples[0:15]),
	  .SAMPLE_VALID_STROBE(sc_sample_valid_strobes[0])
	  );
   
   string_control sc1
	 (
	  .CLK(sc_clk),
	  .RST(sc_rst),
	  .TRIGGER_SAMPLE_STROBE(sc_trigger_sample_strobe),
	  .PLUCK_STROBE(sc_pluck_strobes[1]),
	  .PLUCK_LOCATION(sc_pluck_locations[9:17]),
	  .PLUCK_VALUE(sc_pluck_values[16:31]),
	  .SIZE(sc_sizes[9:17]),
	  .PICKUP_LOC(sc_pickup_locs[8:15]),
	  .SAMPLE(sc_samples[16:31]),
	  .SAMPLE_VALID_STROBE(sc_sample_valid_strobes[1])
	  );
   
   string_control sc2
	 (
	  .CLK(sc_clk),
	  .RST(sc_rst),
	  .TRIGGER_SAMPLE_STROBE(sc_trigger_sample_strobe),
	  .PLUCK_STROBE(sc_pluck_strobes[2]),
	  .PLUCK_LOCATION(sc_pluck_locations[18:26]),
	  .PLUCK_VALUE(sc_pluck_values[32:47]),
	  .SIZE(sc_sizes[18:26]),
	  .PICKUP_LOC(sc_pickup_locs[16:23]),
	  .SAMPLE(sc_samples[32:47]),
	  .SAMPLE_VALID_STROBE(sc_sample_valid_strobes[2])
	  );
   
   string_control sc3
	 (
	  .CLK(sc_clk),
	  .RST(sc_rst),
	  .TRIGGER_SAMPLE_STROBE(sc_trigger_sample_strobe),
	  .PLUCK_STROBE(sc_pluck_strobes[3]),
	  .PLUCK_LOCATION(sc_pluck_locations[27:35]),
	  .PLUCK_VALUE(sc_pluck_values[48:63]),
	  .SIZE(sc_sizes[27:35]),
	  .PICKUP_LOC(sc_pickup_locs[24:31]),
	  .SAMPLE(sc_samples[48:63]),
	  .SAMPLE_VALID_STROBE(sc_sample_valid_strobes[3])
	  );
   

   simulation_control ctrl
	 (
	  .RST(ctrl_rst),
	  .CLK(ctrl_clk),

      .CTRL_EN(CTRL_EN),
      .CTRL_ADDR(CTRL_ADDR),
      .CTRL_DATA_IN(CTRL_DATA_IN),
      .CTRL_DATA_OUT(CTRL_DATA_OUT),
      .CTRL_RNW(CTRL_RNW),
      .CTRL_ACK(CTRL_ACK),

      .MASTER_ENABLE(ctrl_master_enable),
   	  .STRING_LENGTHS(ctrl_string_lengths),
	  .PICKUP_LOCATIONS(ctrl_pickup_locations),
	  .PLUCK_VALUES(ctrl_pluck_values),
	  .PLUCK_LOCATIONS(ctrl_pluck_locations),
	  .PLUCK_STROBES(ctrl_pluck_strobes)
	  );
   

   mixer mix
	 (
	  .RST(mixer_rst),
	  .CLK(mixer_clk),
	  .LEFT_CHANNEL(mixer_left_channel),
	  .RIGHT_CHANNEL(mixer_right_channel),
	  .OUTPUT_VALID_STROBE(mixer_output_valid_strobe),
	  .STRING0_VALID_STROBE(mixer_string0_enable),
	  .STRING1_VALID_STROBE(mixer_string1_enable),
	  .STRING2_VALID_STROBE(mixer_string2_enable),
	  .STRING3_VALID_STROBE(mixer_string3_enable),
	  .STRING0(mixer_string0),
	  .STRING1(mixer_string1),
	  .STRING2(mixer_string2),
	  .STRING3(mixer_string3)
	  );
   
endmodule // top_level
