`timescale 1ns / 1ns

/** tf_nut_filter
 * A test fixture for the nut filter.  The clock runs forever, but
 * the tests only last for a little while.  You'll want to
 * look at the signals in the simulator to see if it's working
 * correctly
 */

module tf_nut_filter();

   reg CLK;
   reg RST;
   reg ENABLE;
   reg [0:15] DATA_IN;
   wire [0:15] DATA_OUT;


   // generate the reset signal
   initial
	 begin
		RST = 1;
		#50; // wait for 2.5 clock cycles with reset high
		RST = 0;
	 end


   // generate the clock signal
   initial
	 begin
		CLK = 1;
		while(1)
		  begin
			 #10;
			 CLK = ~CLK;
		  end
	 end


   // generate the data elements with which to test
   initial
	 begin
		ENABLE = 0;
		DATA_IN = 16'h0000;
		#80; // let the system settle out of the reset state


		// now, we can start giving it data, remember that
		// verilog does not have negatives, so you have to deal
		// with the 2's compliment numbers yourself
		
		DATA_IN = 16'h0200;  
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;

		#40; // wait a couple of clocks for things to settle.  In the
		     // real circuit, this will probably be about 1000 cycles,
		     // but that doesn't affect our testing
	
		DATA_IN = 16'hFFFF;
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;

		#40;

		DATA_IN = 16'h8001;
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;

		#40;

		DATA_IN = 16'h3fff;
		ENABLE = 1;
		#20; // wait a clock cycle
		ENABLE = 0;
				
	 end // initial begin

   nut_filter nf_under_test
	 (
	  .CLK(CLK),
	  .RST(RST),
	  .ENABLE(ENABLE),
	  .DATA_IN(DATA_IN),
	  .DATA_OUT(DATA_OUT)
	  );


endmodule // tf_nut_filter

		
		 
		